\name{go.steep}

\alias{go.steep.default}
\alias{go.steep.mefista}
\alias{mefista}

\title{Steepest descent in space of measures with fixed mass}

\usage{
mef<-go.steep(mu,obj.func,grad.func,
                 obj.func.name=NA,grad.func.name=NA,
                 minim=TRUE,value=NA,grad=NA,tol=0.01,
                 st.size=0.1*sum(mu),algo=1, No.steps=100,space=NA,
                 details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                 draw.style=4,xgrid=NA,ygrid=NA,
                 tmp.file=NA,alpha=NA,base=0.5,
		 reach.only=FALSE,multi.steps=TRUE,
                 uni=FALSE,dim=NA,true.m=NA,nlevels=15,ncolors=0,\dots)

mef<-go.steep(mef,obj.func.name=mef$obj.func$name,
                  grad.func.name=mef$grad.func$name,
                  minim=mef$minim,value=mef$value,
                  grad=mef$grad,tol=mef$param$tol,st.size=mef$st.size,
                  algo=mef$param$algo,No.steps=100,space=mef$param$space,
                  details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                  draw.style=4,xgrid=mef$param$xgrid,ygrid=mef$param$ygrid,
                  tmp.file=mef$param$file,alpha=mef$param$alpha,
                  base=mef$param$base,uni=mef$param$uni,
		  reach.only=mef$reach.only,
		  multi.steps=mef$multi.steps,
                  true.m=mef$true.m,nlevels=15,ncolors=0,\dots)
}

\arguments{
  \item{mu}{
  A vector of the measure weights of all the points of the state space.}
  \item{mef}{
  An object of class \code{mefista} - the result of a previous
  application of method \code{go.steep}.}
  \item{obj.func}{
  The objective function to minimize (maximize). It should have the form 
  \code{<obj.func.name>(mu,space,<other parameters>,\dots)}, where \code{mu} is a
  vector representing the measure, \code{space} is the
  state space as below. Part \code{\dots} should be
  present if you wish to submit additional parameters to the plots. If
  the other parameters are present, they should be listed in
  \code{\dots} part of the \code{go.steep()} call.}
  \item{grad.func}{
  The gradient function corresponding to F=\code{obj.func}, i.e. a
  function of the same form \code{<grad.func.name>(mu,space,<other
    parameters>,\dots)}  as the \code{obj.func} above. It should
  return a vector \code{grad(mu,x)} such that
  \deqn{\lim_{t->+0} t^{-1}[F(\mu+t\eta)-F(\mu)]=\int_{space} grad(\mu,x)
    \eta(dx)}{lim_{t->+0} [F(mu+t*eta)-F(mu)]/t=int_{space} grad(mu,x)
    eta(dx)} for any  measure \eqn{\eta}{eta}. For discrete spaces
  \eqn{\mu=(\mu_1,ldots,\mu_n)}{mu=(mu_1,\dots,mu_n)} and
  \deqn{grad(\mu,x_i)=\frac{d}{d\mu_i}F(\mu)}{grad(mu,x_i)=(d/dmu_i)F(mu).} }
  \item{obj.func.name}{
  The name of the object function. Unless you want it to appear in the
  summary differently, you need not set this parameter: the name will
  be retrieved automatically.}
  \item{grad.func.name}{
  The name of the gradient function, respectively. Retrieved
  automatically.}
  \item{minim}{
  Set this to \code{FALSE} for \emph{maximization} of the objective
  function.}
  \item{fval}{
  The initial value of \code{obj.func}, if known.}
  \item{grad}{
  The initial value of \code{grad.func}, if known.}
  \item{tol}{
  Tolerance: the gradient function is considered to be a constant on a
  set \emph{A} if its variation on \emph{A} is at most tol times the total
  variation of the gradient function. Equally, the objective function is 
  considered to achieve the known \code{true.m} value (see below) if it
  is within \code{tol} error from it.}
  \item{st.size}{
  The initial step size: half of the total variation of the increment
  measure.}
\item{algo}{
  Values: 0 - renormalization, 1 - steepest descent (default), 2 -
  Armijo (see section Details below).
}
  \item{No.steps}{
  Maximal number of steps to go.}
  \item{space}{
  The state space of the measures. A vector or a matrix giving the
  coordinates of the points in each \emph{row} of the latter. If not
  supplied  it is defined as
  \code{\link{expand.grid}(xgrid, ygrid)} if the latter are available,
  otherwise it is considered to be \code{1:length(mu)}.}
  \item{details}{
  If \code{TRUE} additional information on values of the objective
  function, the current step size and the variation of the gradient at
  each step will be printed. Otherwise, the output is a sequence of dots 
  corresponding to steps, "+" and "-" corresponding to increase or
  decrease of the step size, respectively.}
  \item{visualize}{
  If \code{TRUE}, the graph of the obtained measure and the gradient at
  each step will be plotted.}
  \item{smallest.st.size}{
  The computation stops when the value of \code{st.size} becomes less
  than this value.}
  \item{too.small.step.size}{
  Boolean value that the computation has stopped bacause the value of
  \code{st.size} is less than \code{smallest.st.size}.}
  \item{draw.style}{Values: 0 - linear, 1 - \code{\link{image}}, 2 -
    \code{\link{contour}}, 3 - \code{\link{image}} \emph{and} \code{\link{contour}}
    superposed, 4 - \code{\link{persp}}. Additional parameters to plot
  like \code{nlevels} - the number of levels, may be supplied in \code{...}
  part of the call.  If dimension \code{dim} of the \code{space} is not
  2 this parameter is automatically set to 0 - the measure and the
  gradient will be shown on the plot as a linear sequence of values. }
  \item{xgrid, ygrid}{
  In dimension 2 the state \code{space} can be defined as
  \code{\link{expand.grid}(xgrid,ygrid)} - the Cartesian product of the
  vectors. 2D plotting procedures will then be used by default.}
  \item{tmp.file}{A character string (name of the file) to store the
    currently obtained  measure.} 
 \item{alpha}{
   When \code{algo=2}, the parameter of Armijo method (see section Details
   below). Should be between 0 and 1.}
   \item{base}{
   The coefficient the step size is multiplied by when the latter is
   decreased. Should be between 0 and 1.}
   \item{uni}{
   Logical \code{TRUE} or \code{FALSE} - relevant to Algorithm 1 (see
   section Details below).} 
  \item{dim}{
    Dimension of the state space: \code{length(space[1,])}.}
  \item{true.m}{The "true minimal" (maximal) value of \code{obj.func} if 
    known.}
  \item{reach.only}{If \code{TRUE}, the calculation stops as soon
    as  \code{true.m} is reached within a given tolerance \code{tol}. The
    necessary optimality condition is then disregarded.}
  \item{multi.steps}{If \code{FALSE} the programme recalculates the
    gradient after every incremental step. If \code{TRUE} (default) the
    descent (ascent) continues along the gradient while the objective
    function decreses (increases) and only then a new gradient is
    calculated.} 
  \item{nlevels}{
    The number of level lines to draw in 2D drawing procedures.}
  \item{ncolors}{
    The number of colors to use in 2D drawing procedures. You may run
    out of color allocated memory if this parameter is too large. If 0, the
    default color palette is used.}
}
\description{
  Minimization (maximization) of an objective
  function defined on the space of measures with a fixed total mass. For 
optimization with many constraints see function
\code{\link[medea]{go.renorm}} of the library \code{medea}.}

\details{The following algorithms are implemented.
  
  In Algorithm 1 (the default method) incremental measures of
  total variation 2*\code{st.size.cur} and chosen so that to minimize the
  integral of the gradient are added to the current measure mu
  while \code{obj.func} diminishes. Then a new gradient
  is calculated.  If m increments were made at the previous 
  step then the current step size \code{st.size.cur} is increased m
  times. If no increments were possible at the current step size then
  the latter is reduced by \code{base}<1. When constructing the
  increment measure, the algorithm adds a
   positive mass to where the gradient function is minimal.
   If there are a few global minima and \code{uni=TRUE} then the
   mass is uniformly distributed among them. Otherwise the mass is
   added in just one point.

  In Algorithm 2 the current step size is chosen by Armijo method:
  \eqn{st.size.cur=base^k st.size}{st.size.cur=base^k*st.size}, where \eqn{k}{k} is such that
  \deqn{(1) obj.func(mu+eta_k)-obj.func(mu)<=alpha*int_{space} grad(x) eta_k(dx);}{(1) obj.func(mu+eta_k)-obj.func(mu)<=alpha*int_{space} grad(x) eta_k(dx);}
  \deqn{(2) obj.func(mu+eta_{k-1})-obj.func(mu)
  > alpha*int_{space} grad(x) eta_{k-1}(dx).}{(2)
  obj.func(mu+eta_{k-1})-obj.func(mu) 
  > alpha*int_{space} grad(x) eta_{k-1}(dx).}
  Since the value  of the parameter \code{alpha} is critical for the
  speed of 
  convergence it is recommended that you do NOT set it in the first run.
  The default value the program sets is then available in
   \code{<mefista.obj.name>$param$alpha}
   and can be used for subsequent fine tuning of the algorithm.

  Algorithm 0 is the classical renormalization algorithm as described,
  e.g., in 
  Atkinson & Donev "Optimum Experimental Design", Oxford 1992. It adds a 
  positive mass to the minimum point of the gradient function and thus
  obtained measure is then renormalized to the fixed mass. Note that it is
  NOT the steepest descent algorithm, so it is not guaranteed to
  converge.

  The program runs until at least one of the following conditions is
  satisfied: the maximal count of steps is achieved; the step size
  becomes too small; the current measure satisfies the necessary
  optimality condition: the  gradient 
  function is minimal (maximal) and \emph{constant} on the support of the
  optimal measure; the objective function equals the known \code{true.m}
  value  within error \code{tol}. The latter is important,
  e.g., in the case of equation solving: \eqn{F(mu)=C}{F(mu)=C} is
  equivalent to
  minimization of \eqn{(F(mu)-C)^2}{(F(mu)-C)^2} with \code{true.m}=0.

  In computationally extensive cases it is advisable to set
  \code{tmp.file} to
  a name of the file to store the currently obtained
  measure. You then will be able to retrieve it with
  \code{mu<-dget("<name.of.tmp.file>")} and reuse it if the program
  crushes or needs to be interrupted.
}


\value{Returns an object of class \code{mefista} with the following
  named components: 
  \item{mes}{current measure.}
  \item{obj.func}{list with 2 components:}
  \item{obj.func$name}{name of the objective function,}
  \item{obj.func$code}{objective function itself.}
  \item{grad.func}{list with 2 components:}
  \item{grad.func$name}{name of the gradient function,}
  \item{grad.func$code}{gradient function itself.}
  \item{minim}{\code{TRUE} for minimization problem, \code{FALSE}
    otherwise.}
  \item{fval}{value of the objective function.}
  \item{grad}{value of the gradient function.}
  \item{var.grad}{total variation of the gradient on the support of
    the measure.}
  \item{optim}{\code{TRUE} if optimum is attained, \code{FALSE}
    otherwise.}
  \item{st.size}{last value of the step size (the total variation of 
    the increment measure).}
  \item{atoms}{formatted character string giving the atoms of
    the measure. Use \code{cat(<mefista.object>$atoms)} to see them.}
  \item{No.atoms}{number of the measure's atoms.}
  \item{tot.steps}{total number of steps made during the last call.}
  \item{true.m}{value of the true minimum (maximum), if known.}
  \item{reach.only}{logical: whether to stop computation when
    \code{true.m} is reached.}
  \item{multi.steps}{logical: whether to recalculate the gradient after
    each step.}
  \item{param}{list with the following components:}
  \item{param$tol}{tolerance,}
  \item{param$algo}{applied algorithm,}
  \item{param$init.st.size}{initial step size,}
  \item{param$smallest.st.size}{smallest step size,}
  \item{param$uni}{logical value of parameter uni,}
  \item{param$space}{matrix of the state space,}
  \item{param$xgrid}{vector xgrid,}
  \item{param$ygrid}{vector ygrid,}
  \item{param$dim}{dimension of the state space,}
  \item{param$alpha}{parameter of Armijo algorithm,}
  \item{param$base}{parameter of the step size decrease,}
  \item{param$file}{name of the file used to store the obtained
    measure.}
  \item{too.small.step.size}{TRUE if the last computation stopped
    because of the last step size was smaller than \code{param$smallest.st.size},}
  \item{other}{other parameters given in the \code{\dots} part of the call.}
}

\seealso{\code{\link[mefista]{summary.mefista}},
\code{\link[mefista]{plot.mefista}} and
  \code{\link[medea]{go.renorm}} of the library \code{medea}.
}

\examples{
# Optimal design of observation points x_i in linear model
# y(x_i)=\sum_i f(x_i)*beta
# f is a vector function (row) and beta is a column-vector of unknown coeffs.
# Example from Atkinson & Donev "Optimum Experimental Design",
# Oxford 1992 (p.119)
#
\dontrun{library(mefista)
info.matrix <- function(mu,space,func,...)
{ # Information matrix
	return(t(func(space)) \%*\% diag(mu) \%*\% func(space))
}
detIM.func<-function(mu,space,func,...)
  { # The object function: determinant of the info matrix
       return(det(info.matrix(mu,space,func))) 
  }
detIM.grad <- function(mu,space,func,...)
  { # The gradient function (see [1] for details):
    return( diag(func(space) \%*\% solve(info.matrix(mu,space,func)) \%*\% t(func(space))))
  }

 x<-seq(0,1,by=0.01) # State space
 f1<-function(x) {return(x)}
 f2<-function(x) {return(x^2)}
 f3<-function(x) {return(x^3)}
 f<-function(x) {return(cbind(f1(x),f2(x),f3(x)))}
 mu<-rep(1/length(x),length(x)) # Initial measure

 # And here we go.steep:
 res<-go.steep(mu,detIM.func,detIM.grad,minim=FALSE,tol=0.005,space=x,
      algo=2,No.steps=10,details=TRUE,visualize=TRUE,func=f)
 res

 while(!res$opt && !res$too.small.step.size) res<-go.steep(res)
 # The gradient function should take maximal values on
 # the support of the optimal measure:
 plot(res)
 # compare to the analytic result: the optimal measure is uniform with
 # 3 atoms: (5-sqrt(5))/10, (5+sqrt(5))/10 and 1.
 summary(res)}
}

\author{Ilya Molchanov, E-mail: \email{ilya@stat.unibe.ch}\cr
  Sergei Zuyev, E-mail: \email{sergei@stams.strath.ac.uk}}

\references{
  [1] I. Molchanov and S. Zuyev. Steepest descent algorithms in space of
  measures. \emph{Statistics and Computing}, \bold{12}, 115-123, 2002.\cr  
  \url{http://www.cx.unibe.ch/~ilya}\cr
  \url{http://www.stams.strath.ac.uk/~sergei}}

\keyword{optimize}
\keyword{design}
